@@ -27,6 +27,8 @@ ACTION_LOCK_LOCKING = "locking"
 ACTION_LOCK_UNLOCK = "unlock"
 ACTION_LOCK_UNLOCKING = "unlocking"
 ACTION_LOCK_JAMMED = "jammed"
+ACTION_HOMEKEY_LOCK = "homekey_lock"
+ACTION_HOMEKEY_UNLOCK = "homekey_unlock"
 
 ACTION_DOOR_OPEN = "dooropen"
 ACTION_DOOR_OPEN_2 = "door_open"
@@ -60,6 +62,8 @@ ACTIVITY_ACTIONS_LOCK_OPERATION = {
     ACTION_RF_SECURE,
     ACTION_RF_LOCK,
     ACTION_RF_UNLOCK,
+    ACTION_HOMEKEY_LOCK,
+    ACTION_HOMEKEY_UNLOCK,
     ACTION_LOCK_AUTO_LOCK,
     ACTION_LOCK_ONETOUCHLOCK,
     ACTION_LOCK_ONETOUCHLOCK_2,
@@ -82,6 +86,8 @@ ACTIVITY_TO_FIRST_LAST_NAME = {
     ACTION_RF_SECURE: ("Radio Frequency", "Secure"),
     ACTION_RF_LOCK: ("Radio Frequency", "Lock"),
     ACTION_RF_UNLOCK: ("Radio Frequency", "Unlock"),
+    ACTION_HOMEKEY_LOCK: ("Homekey", "Lock"),
+    ACTION_HOMEKEY_UNLOCK: ("Homekey", "Unlock"),
     ACTION_LOCK_AUTO_LOCK: ("Auto", "Lock"),
     ACTION_LOCK_ONETOUCHLOCK: ("One-Touch", "Lock"),
     ACTION_LOCK_ONETOUCHLOCK_2: ("One-Touch", "Lock"),
@@ -114,6 +120,8 @@ TAG_ACTIONS = {
     ACTION_RF_SECURE,
     ACTION_RF_LOCK,
     ACTION_RF_UNLOCK,
+    ACTION_HOMEKEY_LOCK,
+    ACTION_HOMEKEY_UNLOCK,
 }
 
 MANUAL_ACTIONS = {
@@ -125,6 +133,8 @@ ACTIVITY_ACTION_STATES = {
     ACTION_RF_SECURE: LockStatus.LOCKED,
     ACTION_RF_LOCK: LockStatus.LOCKED,
     ACTION_RF_UNLOCK: LockStatus.UNLOCKED,
+    ACTION_HOMEKEY_LOCK: LockStatus.LOCKED,
+    ACTION_HOMEKEY_UNLOCK: LockStatus.UNLOCKED,
     ACTION_LOCK_AUTO_LOCK: LockStatus.LOCKED,
     ACTION_LOCK_ONETOUCHLOCK: LockStatus.LOCKED,
     ACTION_LOCK_ONETOUCHLOCK_2: LockStatus.LOCKED,
