@@ -1,11 +1,25 @@
 """Api calls for sync."""
 
+from __future__ import annotations
+
 import asyncio
+from http import HTTPStatus
 import logging
+from typing import Any
 
-from aiohttp import ClientResponseError, ServerDisconnectedError
+from aiohttp import (
+    ClientConnectionError,
+    ClientOSError,
+    ClientResponse,
+    ClientResponseError,
+    ClientSession,
+    ClientSSLError,
+    ServerDisconnectedError,
+)
 
-from yalexs.api_common import (
+from .activity import ActivityTypes
+from .api_common import (
+    API_EXCEPTION_RETRY_TIME,
     API_LOCK_ASYNC_URL,
     API_LOCK_URL,
     API_RETRY_ATTEMPTS,
@@ -13,6 +27,7 @@ from yalexs.api_common import (
     API_STATUS_ASYNC_URL,
     API_UNLOCK_ASYNC_URL,
     API_UNLOCK_URL,
+    HEADER_ACCEPT_VERSION,
     HEADER_AUGUST_ACCESS_TOKEN,
     HYPER_BRIDGE_PARAM,
     ApiCommon,
@@ -22,26 +37,57 @@ from yalexs.api_common import (
     _process_doorbells_json,
     _process_locks_json,
 )
-from yalexs.doorbell import DoorbellDetail
-from yalexs.exceptions import AugustApiAIOHTTPError
-from yalexs.lock import LockDetail, determine_door_state, determine_lock_status
-from yalexs.pin import Pin
+from .const import DEFAULT_BRAND
+from .doorbell import Doorbell, DoorbellDetail
+from .exceptions import AugustApiAIOHTTPError
+from .lock import (
+    Lock,
+    LockDetail,
+    LockDoorStatus,
+    LockStatus,
+    determine_door_state,
+    determine_lock_status,
+)
+from .pin import Pin
 
 _LOGGER = logging.getLogger(__name__)
 
 
+def _obscure_payload(payload: dict[str, Any]) -> dict[str, Any]:
+    """Obscure the payload for logging."""
+    if payload is None:
+        return None
+    if "password" in payload:
+        payload = payload.copy()
+        payload["password"] = "****"  # nosec
+    return payload
+
+
 class ApiAsync(ApiCommon):
-    def __init__(self, aiohttp_session, timeout=10, command_timeout=60):
+    """Async api."""
+
+    def __init__(
+        self,
+        aiohttp_session: ClientSession,
+        timeout=10,
+        command_timeout=60,
+        brand=DEFAULT_BRAND,
+    ) -> None:
         self._timeout = timeout
         self._command_timeout = command_timeout
         self._aiohttp_session = aiohttp_session
+        super().__init__(brand)
 
-    async def async_get_session(self, install_id, identifier, password):
+    async def async_get_session(
+        self, install_id: str, identifier: str, password: str
+    ) -> ClientResponse:
         return await self._async_dict_to_api(
             self._build_get_session_request(install_id, identifier, password)
         )
 
-    async def async_send_verification_code(self, access_token, login_method, username):
+    async def async_send_verification_code(
+        self, access_token: str, login_method: str, username: str
+    ) -> ClientResponse:
         return await self._async_dict_to_api(
             self._build_send_verification_code_request(
                 access_token, login_method, username
@@ -49,50 +95,60 @@ class ApiAsync(ApiCommon):
         )
 
     async def async_validate_verification_code(
-        self, access_token, login_method, username, verification_code
-    ):
+        self,
+        access_token: str,
+        login_method: str,
+        username: str,
+        verification_code: str,
+    ) -> ClientResponse:
         return await self._async_dict_to_api(
             self._build_validate_verification_code_request(
                 access_token, login_method, username, verification_code
             )
         )
 
-    async def async_get_doorbells(self, access_token):
+    async def async_get_doorbells(self, access_token: str) -> Doorbell:
         response = await self._async_dict_to_api(
             self._build_get_doorbells_request(access_token)
         )
         return _process_doorbells_json(await response.json())
 
-    async def async_get_doorbell_detail(self, access_token, doorbell_id):
+    async def async_get_doorbell_detail(
+        self, access_token: str, doorbell_id: str
+    ) -> DoorbellDetail:
         response = await self._async_dict_to_api(
             self._build_get_doorbell_detail_request(access_token, doorbell_id)
         )
         return DoorbellDetail(await response.json())
 
-    async def async_wakeup_doorbell(self, access_token, doorbell_id):
+    async def async_wakeup_doorbell(
+        self, access_token: str, doorbell_id: str
+    ) -> ClientResponse:
         await self._async_dict_to_api(
             self._build_wakeup_doorbell_request(access_token, doorbell_id)
         )
         return True
 
-    async def async_get_user(self, access_token):
+    async def async_get_user(self, access_token: str) -> dict[str, Any]:
         response = await self._async_dict_to_api(
             self._build_get_user_request(access_token)
         )
         return await response.json()
 
-    async def async_get_houses(self, access_token):
+    async def async_get_houses(self, access_token: str) -> ClientResponse:
         return await self._async_dict_to_api(
             self._build_get_houses_request(access_token)
         )
 
-    async def async_get_house(self, access_token, house_id):
+    async def async_get_house(self, access_token: str, house_id: str) -> dict[str, Any]:
         response = await self._async_dict_to_api(
             self._build_get_house_request(access_token, house_id)
         )
         return await response.json()
 
-    async def async_get_house_activities(self, access_token, house_id, limit=8):
+    async def async_get_house_activities(
+        self, access_token: str, house_id: str, limit: int = 8
+    ) -> list[ActivityTypes]:
         response = await self._async_dict_to_api(
             self._build_get_house_activities_request(
                 access_token, house_id, limit=limit
@@ -100,24 +156,28 @@ class ApiAsync(ApiCommon):
         )
         return _process_activity_json(await response.json())
 
-    async def async_get_locks(self, access_token):
+    async def async_get_locks(self, access_token: str) -> list[Lock]:
         response = await self._async_dict_to_api(
             self._build_get_locks_request(access_token)
         )
         return _process_locks_json(await response.json())
 
-    async def async_get_operable_locks(self, access_token):
+    async def async_get_operable_locks(self, access_token: str) -> list[Lock]:
         locks = await self.async_get_locks(access_token)
 
         return [lock for lock in locks if lock.is_operable]
 
-    async def async_get_lock_detail(self, access_token, lock_id):
+    async def async_get_lock_detail(
+        self, access_token: str, lock_id: str
+    ) -> LockDetail:
         response = await self._async_dict_to_api(
             self._build_get_lock_detail_request(access_token, lock_id)
         )
         return LockDetail(await response.json())
 
-    async def async_get_lock_status(self, access_token, lock_id, door_status=False):
+    async def async_get_lock_status(
+        self, access_token: str, lock_id: str, door_status=False
+    ) -> LockStatus:
         response = await self._async_dict_to_api(
             self._build_get_lock_status_request(access_token, lock_id)
         )
@@ -132,8 +192,8 @@ class ApiAsync(ApiCommon):
         return determine_lock_status(json_dict.get("status"))
 
     async def async_get_lock_door_status(
-        self, access_token, lock_id, lock_status=False
-    ):
+        self, access_token: str, lock_id: str, lock_status=False
+    ) -> LockDoorStatus | tuple[LockDoorStatus, LockStatus]:
         response = await self._async_dict_to_api(
             self._build_get_lock_status_request(access_token, lock_id)
         )
@@ -147,7 +207,7 @@ class ApiAsync(ApiCommon):
 
         return determine_door_state(json_dict.get("doorState"))
 
-    async def async_get_pins(self, access_token, lock_id):
+    async def async_get_pins(self, access_token: str, lock_id: str) -> list[Pin]:
         response = await self._async_dict_to_api(
             self._build_get_pins_request(access_token, lock_id)
         )
@@ -155,7 +215,9 @@ class ApiAsync(ApiCommon):
 
         return [Pin(pin_json) for pin_json in json_dict.get("loaded", [])]
 
-    async def _async_call_lock_operation(self, url_str, access_token, lock_id):
+    async def _async_call_lock_operation(
+        self, url_str: str, access_token: str, lock_id: str
+    ) -> dict[str, Any]:
         response = await self._async_dict_to_api(
             self._build_call_lock_operation_request(
                 url_str, access_token, lock_id, self._command_timeout
@@ -163,7 +225,9 @@ class ApiAsync(ApiCommon):
         )
         return await response.json()
 
-    async def _async_call_async_lock_operation(self, url_str, access_token, lock_id):
+    async def _async_call_async_lock_operation(
+        self, url_str: str, access_token: str, lock_id: str
+    ) -> str:
         """Call an operation that will queue."""
         response = await self._async_dict_to_api(
             self._build_call_lock_operation_request(
@@ -172,12 +236,12 @@ class ApiAsync(ApiCommon):
         )
         return await response.text()
 
-    async def _async_lock(self, access_token, lock_id):
+    async def _async_lock(self, access_token: str, lock_id: str) -> str:
         return await self._async_call_lock_operation(
             API_LOCK_URL, access_token, lock_id
         )
 
-    async def async_lock(self, access_token, lock_id):
+    async def async_lock(self, access_token: str, lock_id: str) -> str:
         """Execute a remote lock operation.
 
         Returns a LockStatus state.
@@ -186,7 +250,9 @@ class ApiAsync(ApiCommon):
             (await self._async_lock(access_token, lock_id)).get("status")
         )
 
-    async def async_lock_async(self, access_token, lock_id, hyper_bridge=True):
+    async def async_lock_async(
+        self, access_token: str, lock_id: str, hyper_bridge=True
+    ) -> str:
         """Queue a remote lock operation and get the response via pubnub."""
         if hyper_bridge:
             return await self._async_call_async_lock_operation(
@@ -196,7 +262,9 @@ class ApiAsync(ApiCommon):
             API_LOCK_ASYNC_URL, access_token, lock_id
         )
 
-    async def async_lock_return_activities(self, access_token, lock_id):
+    async def async_lock_return_activities(
+        self, access_token: str, lock_id: str
+    ) -> list[ActivityTypes]:
         """Execute a remote lock operation.
 
         Returns an array of one or more yalexs.activity.Activity objects
@@ -208,12 +276,12 @@ class ApiAsync(ApiCommon):
             await self._async_lock(access_token, lock_id)
         )
 
-    async def _async_unlock(self, access_token, lock_id):
+    async def _async_unlock(self, access_token: str, lock_id: str) -> dict[str, Any]:
         return await self._async_call_lock_operation(
             API_UNLOCK_URL, access_token, lock_id
         )
 
-    async def async_unlock(self, access_token, lock_id):
+    async def async_unlock(self, access_token: str, lock_id: str) -> LockStatus:
         """Execute a remote unlock operation.
 
         Returns a LockStatus state.
@@ -222,7 +290,9 @@ class ApiAsync(ApiCommon):
             (await self._async_unlock(access_token, lock_id)).get("status")
         )
 
-    async def async_unlock_async(self, access_token, lock_id, hyper_bridge=True):
+    async def async_unlock_async(
+        self, access_token: str, lock_id: str, hyper_bridge=True
+    ) -> str:
         """Queue a remote unlock operation and get the response via pubnub."""
         if hyper_bridge:
             return await self._async_call_async_lock_operation(
@@ -232,7 +302,9 @@ class ApiAsync(ApiCommon):
             API_UNLOCK_ASYNC_URL, access_token, lock_id
         )
 
-    async def async_unlock_return_activities(self, access_token, lock_id):
+    async def async_unlock_return_activities(
+        self, access_token: str, lock_id: str
+    ) -> list[ActivityTypes]:
         """Execute a remote lock operation.
 
         Returns an array of one or more yalexs.activity.Activity objects
@@ -244,7 +316,9 @@ class ApiAsync(ApiCommon):
             await self._async_unlock(access_token, lock_id)
         )
 
-    async def async_status_async(self, access_token, lock_id, hyper_bridge=True):
+    async def async_status_async(
+        self, access_token: str, lock_id: str, hyper_bridge=True
+    ) -> str:
         """Queue a remote unlock operation and get the status via pubnub."""
         if hyper_bridge:
             return await self._async_call_async_lock_operation(
@@ -254,7 +328,7 @@ class ApiAsync(ApiCommon):
             API_STATUS_ASYNC_URL, access_token, lock_id
         )
 
-    async def async_refresh_access_token(self, access_token):
+    async def async_refresh_access_token(self, access_token: str) -> str:
         """Obtain a new api token."""
         return (
             await self._async_dict_to_api(
@@ -262,7 +336,7 @@ class ApiAsync(ApiCommon):
             )
         ).headers[HEADER_AUGUST_ACCESS_TOKEN]
 
-    async def _async_dict_to_api(self, api_dict):
+    async def _async_dict_to_api(self, api_dict: dict[str, Any]) -> ClientResponse:
         url = api_dict["url"]
         method = api_dict["method"]
         access_token = api_dict.get("access_token", None)
@@ -274,26 +348,46 @@ class ApiAsync(ApiCommon):
         payload = api_dict.get("params") or api_dict.get("json")
 
         if "headers" not in api_dict:
-            api_dict["headers"] = _api_headers(access_token=access_token)
+            api_dict["headers"] = _api_headers(
+                access_token=access_token, brand=self.brand
+            )
+
+        if "version" in api_dict:
+            api_dict["headers"][HEADER_ACCEPT_VERSION] = api_dict["version"]
+            del api_dict["version"]
 
         if "timeout" not in api_dict:
             api_dict["timeout"] = self._timeout
 
-        _LOGGER.debug(
-            "About to call %s with header=%s and payload=%s",
-            url,
-            api_dict["headers"],
-            payload,
-        )
+        debug_enabled = _LOGGER.isEnabledFor(logging.DEBUG)
+
+        if debug_enabled:
+            _LOGGER.debug(
+                "About to call %s with header=%s and payload=%s",
+                url,
+                api_dict["headers"],
+                _obscure_payload(payload),
+            )
 
         attempts = 0
-        debug_enabled = _LOGGER.isEnabledFor(logging.DEBUG)
         while attempts < API_RETRY_ATTEMPTS:
             attempts += 1
             try:
                 response = await self._aiohttp_session.request(method, url, **api_dict)
-            except ServerDisconnectedError:
+            except (
+                ClientOSError,
+                ClientSSLError,
+                ServerDisconnectedError,
+                ClientConnectionError,
+            ) as ex:
                 # Try again if we get disconnected
+                # We may get [Errno 104] Connection reset by peer or a
+                # transient disconnect/SSL error
+                if attempts == API_RETRY_ATTEMPTS:
+                    raise AugustApiAIOHTTPError(
+                        f"Failed to connect to August API: {ex}", ex
+                    ) from ex
+                await asyncio.sleep(API_EXCEPTION_RETRY_TIME)
                 continue
             if debug_enabled:
                 _LOGGER.debug(
@@ -308,7 +402,7 @@ class ApiAsync(ApiCommon):
                     "August sent a 429 (attempt: %d), sleeping and trying again",
                     attempts,
                 )
-                asyncio.sleep(API_RETRY_TIME)
+                await asyncio.sleep(API_RETRY_TIME)
                 continue
             break
 
@@ -317,22 +411,30 @@ class ApiAsync(ApiCommon):
         return response
 
 
-def _raise_response_exceptions(response):
+def _raise_response_exceptions(response: ClientResponse) -> None:
+    """Raise exceptions for known error codes."""
     try:
         response.raise_for_status()
     except ClientResponseError as err:
+        if err.status in (HTTPStatus.UNAUTHORIZED, HTTPStatus.FORBIDDEN):
+            raise AugustApiAIOHTTPError(
+                f"Authentication failed; Verify brand is correct: {err.message}", err
+            ) from err
         if err.status == 422:
             raise AugustApiAIOHTTPError(
-                "The operation failed because the bridge (connect) is offline.",
+                f"The operation failed because the bridge (connect) is offline: {err.message}",
+                err,
             ) from err
         if err.status == 423:
             raise AugustApiAIOHTTPError(
-                "The operation failed because the bridge (connect) is in use.",
+                f"The operation failed because the bridge (connect) is in use: {err.message}",
+                err,
             ) from err
         if err.status == 408:
             raise AugustApiAIOHTTPError(
-                "The operation timed out because the bridge (connect) failed to respond.",
+                f"The operation timed out because the bridge (connect) failed to respond: {err.message}",
+                err,
             ) from err
         raise AugustApiAIOHTTPError(
-            f"The operation failed with error code {err.status}: {err.message}.",
+            f"The operation failed with error code {err.status}: {err.message}.", err
         ) from err
